#ifndef _LINUX_HUNGTASK_H
#define _LINUX_HUNGTASK_H

#ifdef CONFIG_FUSIONOS_HUNGTASK_DEBUG

#define HUNGTASK_LOCK_DEPTH  32
struct hungtask_control_ops {
	void (*add_hung_task)(struct task_struct *task, unsigned long timeout);
	void (*check_task_end)(bool full_check);
};

struct lock_info {
	void *lock_addr;
	unsigned long *lock_caller;
};

struct hungtask_info {
	struct lock_info locking_info;
	struct lock_info locked_info[HUNGTASK_LOCK_DEPTH];
	int locked_cnt;
};

extern void register_hungtask_control(struct hungtask_control_ops *co);
extern void unregister_hungtask_control(struct hungtask_control_ops *co);

extern int hungtask_debug_enable;
extern void hungtask_lock_acquire(void *lock_addr, unsigned long *ip);
extern void hungtask_lock_acquire_release(void *lock_addr, struct task_struct *task);
extern void hungtask_lock_acquired(void *lock_addr, unsigned long *ip);
extern void hungtask_lock_acquired_release(void *lock_addr);
#else
#define register_hungtask_control(struct hungtask_control_ops *co) do {} while (0)
#define unregister_hungtask_control(struct hungtask_control_ops *co) do {} while (0)
#define hungtask_lock_acquire(void *lock_addr, unsigned long *ip) do {} while (0)
#define hungtask_lock_acquire_release(void *lock_addr, struct task_struct *task) do {} while (0)
#define hungtask_lock_acquired(void *lock_addr, unsigned long *ip) do {} while (0)
#define hungtask_lock_acquired_release(void *lock_addr) do {} while (0)

#endif
#endif /* _LINUX_HUNGTASK_H */
