/*
 * Copyright (c) Xfusion Technologies Co., Ltd. 2022-2032. All rights reserved.
 * Description: nvram manage for printk and kbox to record debug log
 * Author: wangyoukang
 * Create: 2022.11.07
 */

#ifndef _NVRAM_DEBUG_LOG_H
#define _NVRAM_DEBUG_LOG_H

#ifdef CONFIG_ARM64
#include <asm/page-def.h>
#else
#include <asm/page_types.h>
#endif
#include <linux/sched.h>

/* NVRAM area total size: This should be reported from the bios up, the current size 128M */
#define NVRAM_TOTAL_SIZE (0x8000000)

/* size of printk area 8M*/
#define PRINTK_AREA_SIZE 0x800000

/* reserve for page align */
#define RESERVE_PAD_SIZE                                                       \
	(PAGE_SIZE - (RESERVE_KBOX_SIZE + PAGE_SIZE + sizeof(unsigned long) +  \
		      sizeof(unsigned long) + sizeof(unsigned long) +          \
		      sizeof(unsigned long) + sizeof(unsigned long) +          \
		      sizeof(unsigned long)) %                                 \
			     PAGE_SIZE)

/* reserve for kbox module:16M */
#define RESERVE_KBOX_SIZE (0x1000000)

/* Reboot magic word */
#define NVRAM_MAGIC_NUMBER 0x9A1BB2A3

struct printk_management_nvram {
	unsigned long printk_start;
	unsigned long printk_end;
	unsigned long console_start;
	unsigned long printk_buf_size;
	unsigned long power_flag;
	unsigned long printk_area_backup;
	char reserve_pad_area[RESERVE_PAD_SIZE];
};

/*
 * 1.Re-adjust the internal data structure variables
 * 2.Remove the member variables that are no longer in use
 * 3.Defines the area reserved for other modules
 * 4.When adding new members, make sure 8 bytes are aligned, otherwise there is a problem with the alignment of the compiler alignment
 * Note: This structure is very, very large, can not be directly quoted, can only use the pointer; suggested that follow the optimization of this structure definition
 */
struct nvram_map {
	/* kbox area with protect */
	char reserve_kbox_area[RESERVE_KBOX_SIZE]; /* 16M, for kbox */
	char ro_area0[PAGE_SIZE]; /* 4K read-only */

	struct printk_management_nvram nvram_manage;

	/* printk area with protect */
	char ro_area1[PAGE_SIZE]; /* 4K read-only */
	char printk_area[PRINTK_AREA_SIZE]; /* 8M printk log */
	char ro_area2[PAGE_SIZE]; /* 4K read-only */

	/* reserve area */
	char reserve_area[PAGE_SIZE]; /* 4K reserved area */
};

extern int is_log_nvram_redirect_on(void);

/* stop other CPUs when save data */
extern char *get_kbox_reserve_area(void);

#endif /*_NVRAM_DEBUG_LOG_H*/
